"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
const controller_1 = require("./controller/controller");
class InputWatcher {
    constructor(table) {
        this.gamepad = controller_1.Controllers.getAnyGamepad();
        this.table = table;
        this.polledInput = null;
        this.repeatIntervalInSeconds = null;
        this.boundKeys = [];
        const keyboardKey = (key) => {
            const button = controller_1.Controllers.key(key, table);
            this.boundKeys.push(button);
            return button;
        };
        // const anyGamepad = Contro.getAnyGamepad()
        // const stick = Contro.orSticks([
        //     dpadAsStick(anyGamepad),
        //     anyGamepad.stick(STICK_TYPE.LEFT),
        //     anyGamepad.stick(STICK_TYPE.RIGHT),
        //     // arrowKeysAsStick(),
        //     // wsadKeysAsStick(),
        //     Contro.asStick(keyboardKey('w'), keyboardKey('s'), keyboardKey('a'), keyboardKey('d')),
        //     Contro.asStick(keyboardKey('ArrowUp'), keyboardKey('ArrowDown'), keyboardKey('ArrowLeft'), keyboardKey('ArrowRight')),
        // ])
        // const action = Contro.or([this.gamepad.button(BUTTON_TYPE.CLUSTER_BOTTOM), keyboardKey('x'), keyboardKey(' '), keyboardKey('Enter')])
        // const undo = Contro.or([this.gamepad.button(BUTTON_TYPE.CLUSTER_LEFT), this.gamepad.button(BUTTON_TYPE.BUMPER_TOP_LEFT), keyboardKey('z'), keyboardKey('u')])
        // const restart = Contro.or([this.gamepad.button(BUTTON_TYPE.CLUSTER_TOP), keyboardKey('r')])
        this.controls = {
            up: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.ARROW_UP), keyboardKey('w'), keyboardKey('ArrowUp')]) },
            down: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.ARROW_DOWN), keyboardKey('s'), keyboardKey('ArrowDown')]) },
            left: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.ARROW_LEFT), keyboardKey('a'), keyboardKey('ArrowLeft')]) },
            right: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.ARROW_RIGHT), keyboardKey('d'), keyboardKey('ArrowRight')]) },
            action: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.CLUSTER_BOTTOM), keyboardKey('x'), keyboardKey(' '), keyboardKey('Enter')]) },
            undo: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.CLUSTER_LEFT), this.gamepad.button(controller_1.BUTTON_TYPE.BUMPER_TOP_LEFT), keyboardKey('z'), keyboardKey('u')]) },
            restart: { lastPressed: null, button: controller_1.Controllers.or([this.gamepad.button(controller_1.BUTTON_TYPE.CLUSTER_TOP), keyboardKey('r')]) }
        };
        this.stickControls = {
            left: { lastPressed: null, stick: this.gamepad.stick(controller_1.STICK_TYPE.LEFT) },
            right: { lastPressed: null, stick: this.gamepad.stick(controller_1.STICK_TYPE.RIGHT) }
        };
        const makeChecker = (control, gameAction) => {
            return () => {
                const now = Date.now();
                const interval = (this.repeatIntervalInSeconds || .25) * 1000;
                const { button, lastPressed } = this.controls[control];
                if (button.query()) {
                    if (!lastPressed || (lastPressed + interval) < now) {
                        gameAction();
                        this.controls[control].lastPressed = now;
                    }
                }
                else {
                    this.controls[control].lastPressed = null;
                }
            };
        };
        const makeStickChecker = (whichStick) => {
            return () => {
                const now = Date.now();
                const interval = (this.repeatIntervalInSeconds || .25) * 1000;
                const control = this.stickControls[whichStick];
                const { stick, lastPressed } = control;
                const { x: xAxis, y: yAxis } = stick.query();
                const yLean = Math.abs(yAxis);
                const xLean = Math.abs(xAxis);
                if (yLean > xLean) {
                    if (yLean > .5) {
                        if (!lastPressed || (lastPressed + interval) < now) {
                            control.lastPressed = now;
                            this.polledInput = yAxis < 0 ? util_1.INPUT_BUTTON.UP : util_1.INPUT_BUTTON.DOWN;
                        }
                        return;
                    }
                }
                else {
                    if (xLean > .5) {
                        if (!lastPressed || (lastPressed + interval) < now) {
                            control.lastPressed = now;
                            this.polledInput = xAxis < 0 ? util_1.INPUT_BUTTON.LEFT : util_1.INPUT_BUTTON.RIGHT;
                        }
                        return;
                    }
                }
                control.lastPressed = null;
            };
        };
        this.controlCheckers = [
            makeStickChecker('left'),
            makeStickChecker('right'),
            makeChecker('up', () => this.polledInput = util_1.INPUT_BUTTON.UP),
            makeChecker('down', () => this.polledInput = util_1.INPUT_BUTTON.DOWN),
            makeChecker('left', () => this.polledInput = util_1.INPUT_BUTTON.LEFT),
            makeChecker('right', () => this.polledInput = util_1.INPUT_BUTTON.RIGHT),
            makeChecker('action', () => this.polledInput = util_1.INPUT_BUTTON.ACTION),
            makeChecker('undo', () => this.polledInput = util_1.INPUT_BUTTON.UNDO),
            makeChecker('restart', () => this.polledInput = util_1.INPUT_BUTTON.RESTART)
        ];
    }
    pollControls() {
        if (window.document.activeElement !== this.table) {
            // Ignore if the game is not in focus
            return null;
        }
        this.polledInput = null;
        this.controlCheckers.forEach((fn) => fn());
        const input = this.polledInput;
        this.polledInput = null;
        return input;
    }
    isSomethingPressed() {
        for (const entry of Object.values(this.controls)) {
            if (entry.button.query()) {
                return true;
            }
        }
        return false;
    }
    setKeyRepeatInterval(repeatIntervalInSeconds) {
        this.repeatIntervalInSeconds = repeatIntervalInSeconds;
    }
    dispose() {
        for (const button of this.boundKeys) {
            button.dispose();
        }
    }
}
exports.default = InputWatcher;
//# sourceMappingURL=InputWatcher.js.map